Class: Braid::Mirror::Options
- Defined in:
- lib/braid/mirror.rb
Overview
Mirror::Options doubles as the options struct for the add command, so some properties are meaningful only in that context. TODO: Maybe the code could be organized in a better way.
Method Summary
Methods inherited from T::Struct
Constructor Details
This class inherits a constructor from Braid::T::Struct